import contextlib
import logging

import mysql.connector

from inventory_provider.constants import DATABASE_LOGGER_NAME


@contextlib.contextmanager
def connection(opsdb):  # pragma: no cover
    cx = None
    try:
        cx = mysql.connector.connect(
            host=opsdb["hostname"],
            user=opsdb["username"],
            passwd=opsdb["password"],
            db=opsdb["dbname"])
        yield cx
    finally:
        if cx:
            cx.close()


@contextlib.contextmanager
def cursor(cnx):  # pragma: no cover
    csr = None
    try:
        csr = cnx.cursor()
        yield csr
    finally:
        if csr:
            csr.close()


def _db_test(db, router):
    database_logger = logging.getLogger(DATABASE_LOGGER_NAME)
    with cursor(db) as crs:
        query = "select model, manufacturer from equipment where name = %s"
        crs.execute(query, (router['hostname'],))
        for (model, manufacturer) in crs:
            database_logger.debug("%s: %s %s" % (
                router['hostname'], model, manufacturer))
            yield {"model": model, "manufacturer": manufacturer}
